<?php

class w2dc_review extends w2dc_post {
	public $post;
	public $listing;
	public $images = array();
	public $logo_image = false;
	public $avg_rating;
	public $ratings;
	public $votes = array();
	public $votes_up = 0;
	public $votes_down = 0;
	public $current_user_voted = false;
	
	public function loadReviewFromPost($post) {
		if (!$post) {
			return false;
		}
		
		if ($this->setPost($post)) {
			if ($this->post->post_type == W2DC_REVIEW_TYPE) {
				$this->setListing();
				$this->setMedia();
				$this->setRatings();
				$this->setVotes();
					
				apply_filters('w2dc_review_loading', $this);
			
				return true;
			}
		}
	}
	
	public function setListing() {
		$listing_id = get_post_meta($this->post->ID, '_listing_id', true);
		
		if ($this->listing = w2dc_getListing($listing_id)) {
			return $this->listing;
		}
	}
	
	public function setMedia() {
		if (get_option("w2dc_reviews_images_number")) {
			if ($images = get_post_meta($this->post->ID, '_attached_image')) {
				if ($images_order = explode(',', get_post_meta($this->post->ID, '_attached_images_order', true))) {
					$images = array_flip(array_replace(array_flip(array_unique(array_filter($images_order))), array_flip($images)));
				}
				foreach ($images AS $image_id) {
					// adapted for WPML
					global $sitepress;
					if (function_exists('wpml_object_id_filter') && $sitepress)
						$image_id = apply_filters('wpml_object_id', $image_id, 'attachment', true);
	
					if ($image_post = get_post($image_id, ARRAY_A)) {
						$this->images[$image_id] = $image_post;
					}
				}
	
				/* if (($logo_id = (int)get_post_meta($this->post->ID, '_attached_image_as_logo', true)) && in_array($logo_id, array_keys($this->images))) {
					$this->logo_image = $logo_id;
				} else {
					$images_keys = array_keys($this->images);
					$this->logo_image = array_shift($images_keys);
				} */
			} else {
				$this->images = array();
			}
		}
	
		$this->images = apply_filters('w2dc_review_images', $this->images, $this);
	}
	
	public function setRatings() {
		if (!($avg_value = get_post_meta($this->post->ID, '_avg_rating', true))) {
			$avg_value = 5;
		}
		$this->avg_rating = new w2dc_review_rating($avg_value);
		
		$this->ratings = get_post_meta($this->post->ID, '_review_ratings', true);
	}
	
	public function setVotes() {
		global $wpdb, $w2dc_instance;
		
		$like = $wpdb->esc_like(W2DC_REVIEW_VOTE_PREFIX);
		
		$results = $wpdb->get_results($wpdb->prepare("SELECT DISTINCT meta_key, meta_value FROM {$wpdb->postmeta} WHERE post_id=%d AND meta_key LIKE %s", $this->post->ID, $like.'%'), ARRAY_A);
		foreach ($results AS $row) {
			$this->votes[] = $row;
			if ($row['meta_value']) {
				$this->votes_up++;
			} else {
				$this->votes_down++;
			}
		}
		
		if ($w2dc_instance->reviews_manager->is_review_voted($this->post->ID)) {
			if ($user_id = get_current_user_id()) {
				$meta = get_post_meta($this->post->ID, W2DC_REVIEW_VOTE_PREFIX . $user_id, true);
				if ($meta != '') {
					$this->current_user_voted = intval($meta);
					return ;
				}
			}
			if ($ip = w2dc_ip_address()) {
				$meta = get_post_meta($this->post->ID, W2DC_REVIEW_VOTE_PREFIX . $ip, true);
				if ($meta != '') {
					$this->current_user_voted = intval($meta);
				}
			}
		}
	}
	
	public function renderStars() {
		if ($this->avg_rating) {
			w2dc_renderTemplate(array(W2DC_RATINGS_TEMPLATES_PATH, 'single_rating.tpl.php'), array('rating' => $this->avg_rating));
		}
	}
	
	public function display($frontend_controller, $is_single = false) {
		$template = array(W2DC_RATINGS_TEMPLATES_PATH, 'review.tpl.php');
	
		$template = apply_filters('w2dc_review_display_template', $template, $is_single, $this);
	
		w2dc_renderTemplate($template, array('frontend_controller' => $frontend_controller, 'review' => $this, 'is_single' => $is_single));
	}
}

?>